#!/usr/bin/perl
# ex: set tabstop=4 expandtab:

# $Id: extended_cpu.pl 30171 2009-06-22 17:58:48Z bgunn $

# extended_cpu.pl  by Billy Gunn (billy@gunn.org)

# Queries raw cpu counters via snmp. The couters are adjusted
# to represent roughly 100% total usage. The adjustment is done
# by calculating the number of actual time ticks over the elapsed
# time and dividing by the multiple. For instance a quad cpu quad 
# core server will have 1600 cpu ticks per second. If the numbers 
# were not adjusted cpu usage would appear to go as high as 1600%.

use strict;
use SNMP;
use Getopt::Long;

use vars qw($opt_H $opt_C $opt_v $opt_p);

MAIN: {

    Getopt::Long::Configure('bundling');
    my $status = GetOptions (
        "H=s" => \$opt_H,  "host=s"      => \$opt_H,
        "C=s" => \$opt_C,  "Community=s" => \$opt_C,
        "v=s" => \$opt_v,  "version=s"   => \$opt_v,
        "p=s" => \$opt_p,  "port=s"      => \$opt_p
    );

    $opt_v = '2c';

    my $procs = 1;
    my $time = time();
    my $file = "/tmp/$opt_H.extended_cpu.txt";

    my ($user, $nice, $sys, $idle, $wait, $irq, $softirq, $kernel) = 0;

    # Create an SNMP session
    my $session = new SNMP::Session (
        DestHost    => $opt_H,
        Community   => $opt_C,
        RemotePort  => $opt_p,
        Version     => $opt_v
    );

    if ( $session->{ErrorNum} ) {
            die "Error retrieving SNMP data: $session->{ErrorStr}\n";
    }

    ($user, $nice, $sys, $idle, $wait, $irq, $softirq, $kernel) = $session->bulkwalk( 0, 8,
            [
             ['ssCpuRawUser'],
             ['ssCpuRawNice'],
             ['ssCpuRawSystem'],
             ['ssCpuRawIdle'],
             ['ssCpuRawWait'],
             ['ssCpuRawInterrupt'],
             ['ssCpuRawSoftIRQ'],
             ['ssCpuRawKernel']
            ]
    );

    alarm(0);

    # Grab the values from the arrays
    $user    = scalar(@$user[0]->val)    if (@$user[0]);
    $sys     = scalar(@$sys[0]->val)     if (@$sys[0]);
    $idle    = scalar(@$idle[0]->val)    if (@$idle[0]);
    $irq     = scalar(@$irq[0]->val)     if (@$irq[0]);
    $softirq = scalar(@$softirq[0]->val) if (@$softirq[0]);
    $kernel  = scalar(@$kernel[0]->val)  if (@$kernel[0]);
    $wait    = scalar(@$wait[0]->val)    if (@$wait[0]);
    $nice    = scalar(@$nice[0]->val)    if (@$nice[0]);

    my ($ltime, $luser, $lnice, $lsys, $lidle, $lwait, $lirq, $lsoftirq);

    if (-e $file) {
        open(FILE, "< $file") or die "Can't open file $opt_H.extended_cpu.txt: $!\n";
        while(<FILE>) {
            chomp;
            ($ltime, $luser, $lnice, $lsys, $lidle, $lwait, $lirq, $lsoftirq) = split(/ /, $_);
        }
        close(FILE);
    }

    open(FILE, "> $file") or die "Can't open file $opt_H.extended_cpu.txt: $!\n";
    print FILE "$time $user $nice $sys $idle $wait $irq $softirq";
    close(FILE);

    exit if (!$ltime);

    my $total = $user + $nice + $sys + $idle + $softirq + $irq + $wait;
    my $ltotal = $luser + $lnice + $lsys + $lidle + $lsoftirq + $lirq + $lwait;

    # Correct for the number of processors
    my $procs = int(((($total - $ltotal) / ($time - $ltime)) / 100) + .5);

    $user    = int(($user    / $procs) + .5);
    $sys     = int(($sys     / $procs) + .5);
    $idle    = int(($idle    / $procs) + .5);
    $irq     = int(($irq     / $procs) + .5);
    $softirq = int(($softirq / $procs) + .5);
    $wait    = int(($wait    / $procs) + .5);
    $nice    = int(($nice    / $procs) + .5);

    print "user:$user nice:$nice sys:$sys idle:$idle wait:$wait irq:$irq softirq:$softirq\n";
}

